﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

	Function SeparateDataByCore()
		SetDataFolder root:
			String/G G_Hole, G_Folder, G_DepthType
			String Hole = G_Hole, Folder = G_Folder, NumWvList, StrWvList, AllWvList
			String NewWaveName, DepthType = G_DepthType, Technique
			Variable Core, NumberOfCores, i, j, FirstUS
			Prompt Hole, "Hole:"
			Prompt DepthType, "Depth Type ( mbsf, csf, etc ):"
			Prompt Folder, "Select Data Folder:", popup, FolderList("*")
			DoPrompt "SeparateDataByCore", Hole, DepthType, Folder
			Hole = UpperStr( Hole )
			G_Hole = Hole; G_Folder = Folder; G_DepthType = DepthType
			AbortOnValue V_Flag, 0
			Print "SeparateDataByCore_F( \"" + Hole + "\", \"" + DepthType + "\", \"" + Folder + "\" )"
			SeparateDataByCore_F( Hole, DepthType, Folder )
	End

-------------------------------------------------------------------------------------------------	

	Function SeparateDataByCore_F( Hole, DepthType, Folder )
			String Hole, DepthType, Folder
		SetDataFolder root:
		DepthType = LowerStr( DepthType )
			String NumWvList, StrWvList, AllWvList
			String NewWaveName, Technique
			Variable Core, NumberOfCores, i, j, FirstUS
			Hole = UpperStr( Hole )
			Technique = SecondPart( Folder )  // The Technique is the second part of Folder name. "SecondPart()" is a CODD function
			CheckEqualWaveLengthsHole( Hole, SecondPart( Folder ) ) // A CODD check routine to make sure all of the waves to be processed are of the same length ( necessary for sorting )
			WAVE Matrix = $Hole + "_Coring_Matrix"  // Set the Coring Matrix
			If( !WaveExists( Matrix ) )  // Check to see if the Coring Matrix is loaded and in root:
				ABORT "I can't find \"" + Hole + "_Coring_Matrix\""
			Endif
			NumberOfCores = DimSize( Matrix, 0 ) // Gets the number of cores from the Coring Matrix
		NewDataFolder/O $Hole + "_" + SecondPart( Folder ) + "ByCore"  // Make a new data folder to hold the core-separated data waves
			DFREF Output = $Hole + "_" + SecondPart( Folder ) + "ByCore"  // Set a reference for the new folder
		SetDataFolder $Folder  // Move the pointer to the folder holding the data to be split into cores
			NumWVList = WaveList( Hole + "*", ";", "TEXT:0" ) // Make a wave list of numerical waves
			StrWVList = WaveList( Hole + "*", ";", "TEXT:1" ) // Make a list of text waves
			WAVE CoreWv = $Hole + "_" + Technique + "_Core"  // Identifies the depth wave for the Hole of interest
			Duplicate/O CoreWv root:MASK  // Create a MASK that is a duplicate of the Hole core wave
			WAVE MASK = root:MASK  // Set the wave reference
			For( Core = 1; Core < NumberOfCores; Core += 1 )  // This loop does the work of separating the data 1 core at a time
				MASK = ( CoreWv == Core )  // The MASK is set to 1 when the the values in the core wave equal the core number being processed in the loop
				If( Sum( MASK ) == 0 )  // If the sum of all the values in the MASK is 0, there are no data for the core being processed 
					Continue  // "Continue" drops processing to the end of the loop for this core, but does not stop the loop
				EndIf
				For( i = 0; i < ItemsInList( NumWvList ); i += 1 )  // This loop processes the numerical waves
					WAVE Data = $StringFromList( i, NumWvList )
					FirstUS = StrSearch( NameOfWave( Data ), "_", 0 ) // Look for the postion of the first underscore in the data wave name
					If( Core < 10 )
						NewWaveName = NameOfWave( Data )[ 0, FirstUS - 1 ] + "0" + Num2Str( Core ) + NameOfWave( Data )[ FirstUS, inf ] // Make a new name that inserts the core number after Hole
					Else
						NewWaveName = NameOfWave( Data )[ 0, FirstUS - 1 ] + Num2Str( Core ) + NameOfWave( Data )[ FirstUS, inf ] // Make a new name that inserts the core number after Hole
					EndIf
					Duplicate/O Data Output:$NewWaveName // Copies the data wave into the output folder with the new core wave name
					WAVE NewWave = OutPut:$NewWaveName
					NewWave = MASK == 0 ? NaN : NewWave  // The core wave is set to NaN for data that are not within the core
				EndFor
				For( i = 0; i < ItemsInList( StrWvList ); i += 1 )  // Make new text waves for the core, same as above
					WAVE/S Data = $StringFromList( i, StrWvList )
					FirstUS = StrSearch( NameOfWave( Data ), "_", 0 ) // Find out where the Hole designation ends in the wave name
					If( Core < 10 )
						NewWaveName = NameOfWave( Data )[ 0, FirstUS - 1 ] + "0" + Num2Str( Core ) + NameOfWave( Data )[ FirstUS, inf ] // Make a new name that inserts the core number after Hole
					Else
						NewWaveName = NameOfWave( Data )[ 0, FirstUS - 1 ] + Num2Str( Core ) + NameOfWave( Data )[ FirstUS, inf ] // Make a new name that inserts the core number after Hole
					EndIf
					Duplicate/O Data Output:$NewWaveName
				EndFor
			 SetDataFolder OutPut	// Change to the output folder			 
			 		WAVE DepthWave = $FirstPart( NewWaveName ) + "_" + SecondPart( NewWaveName ) + "_" + DepthType // Set up the depth wave as a wave ref
					AllWvList = WaveList( Hole + "*" + Num2Str( Core ) + "_*", ",", "" )  // Make a list of all of the waves for the core being processed
					String Exec1 = "Sort " + NameOfWave( DepthWave ) + "," + AllWvList // Sort all of the waves according to depth
					EXECUTE Exec1
					WaveStats/Q DepthWave  // Use wavestats to find out how many data points ( vs NaNs ) are in the core waves
					EXECUTE "Redimension/N = " + Num2Str( V_Npnts) + " " + AllWvList  // Redimension all of the waves to only be as long as the number of data points
			 SetDataFolder $"root:" + Folder	// Back to the data folder
			EndFor
		SetDataFolder root:
			KillWaves/Z MASK  // Get rid of the mask. The "/Z" does not work in older Igor versions
	End
	
