﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

	Function SingleHoleImage()
		SetDataFolder root:
			String/G G_Folder
			String Folder = G_Folder
			Prompt Folder, "Core Image Folder", popup, FolderList( "*" )
			DoPrompt "SingleHoleImage", Folder
			G_Folder = Folder; AbortOnValue V_Flag, 0
			DFREF ImageFldr = $Folder
			Make/O/N = ( 100, 101, 3 ) $FirstPart( Folder ) + "_Image" // Make a new imagein root: that will later be redimensioned to the proper size
			WAVE HoleImage = $FirstPart( Folder ) + "_Image" // The new image in root: will be named after the Hole plus "_Image"
		SetDataFolder ImageFldr // Move to the image folder
			PauseUpdate
			String WvList
			Variable Items, TopDepth, BottomDepth, XResolution, YResolution, XPoints, YPoints, XLength
			WvList = SortList( WaveList( "*", ";", "DIMS:3" ) ) // Create a SORTED list of the image waves in the folder
			Items = ItemsInList( WvList ) // Items is the number of images in the folder
			WAVE CoreImage = $StringFromList( 0, WvList ) // Set a waveref for the first image in the list to get needed resolution and size data
			XResolution = DimDelta( CoreImage, 0 ) // Get the depth resolutiopn of the image
			YResolution = DimDelta( CoreImage, 1 ) // Get the width resolutiopn of the image
			XPoints = DimSize( CoreImage, 0 ) // Get the length ( number of points in the X direction )
			YPoints = DimSize( CoreImage, 1 ) // Get the width ( number of points in the Y direction
			TopDepth = LeftX( $StringFromList( 0, WvList ) ) // Get the top depth from the top of the first ( shallowest ) core image
			BottomDepth = RightX( $StringFromList( Items - 1, WvList ) ) // Get the bottom depth from the bottom depth of the last ( deepest ) core image
			XLength = ( BottomDepth - TopDepth ) / XResolution; XLength += 1 // Calculate how long ( in points ) the Hole image needs to be and pad with an extra point to avoid possible "index out of range" errors
			Redimension/B/U/N = ( XLength, YPoints, 3 ) HoleImage // Redimension the Hole image to the proper length in points
			SetScale/P x TopDepth, XResolution, "" , HoleImage // Set the X scale. Use the Start and Delta method ( "/P" )
			SetScale/P y 0, YResolution, "", HoleImage // Set the Y scale. Use the Start and Delta method ( "/P" )
			HoleImage = NaN // Set the Hole image to NaN
			Variable i
			For( i = 0; i < Items; i += 1 ) // This loop adds the core images to the Hole image in order from the wavelist of core images
				WAVE CoreImage = $StringFromList( i, WvList )
				HoleImage[ X2Pnt( HoleImage, LeftX( CoreImage ) ), X2Pnt( HoleImage, RightX( CoreImage ) - 1 ) ][][] = CoreImage[ p - X2Pnt( HoleImage, LeftX( CoreImage ) ) ][q][r] // Complicated - see below
			EndFor			
		SetDataFolder root:
			Variable err = GetRTError(1) // This line suppresses error messages - specifically to avoid indexing problems
	End
	
// In order to copy an image onto a section of a larger image you have use points, not x or y. 
//  HoleImage[ X point on the hole image that is the top of the core image depth, X point on the hole image that is the bottom of the core image depth ][][]
//				= CoreImage[ p - X point on the hole image that is the top of the core image depth ][][]//
// The "p" above starts with the same value as the top X point of the HoleImage you want to paste into.
//				By subtracting that value from p, the copying starts at the 0 point of the core image. Using p
//				means that all points are copied from top to bottom.
// The [][] at the end of the expression on each side of the code line helps to aviod index problems because the
//				length of the Hole image is rarely evenly divisable by the X resolution.