﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

	Function SpectralAgeDepthFit( )
		SetDataFolder root:
			PauseUpdate
			String/G G_Folder, G_AgeWave, G_DepthWave, G_ZeroYorN, G_Site, G_CursorYorN, G_AgeOrDepth
			String Folder = G_Folder, AgeWave = G_AgeWave, DepthWave = G_DepthWave, AgeOrDepth = G_AgeOrDepth
			String CursorYorN = G_CursorYorN, ZeroYorN = G_ZeroYorN, Site = G_Site
			Variable/G G_PolyOrder
			Variable PolyOrder = G_PolyOrder			
			Prompt Site, "Site ( e.g. U1338 )"
			Prompt Folder, "Select the Data Folder with the Age-Depth Data", popup, "root:;" + SortList( FolderList( "*" ) )
			DoPrompt "AgeDepthFit", Site, Folder;
			G_Folder = Folder; G_Site = Site
			AbortOnValue V_Flag, 0
		SetDataFolder $Folder
			Prompt DepthWave, "DepthWave", popup, SortList( WaveList( "*", ";", "" ) )
			Prompt AgeWave, "AgeWave", popup, SortList( WaveList( "*", ";", "" ) )
			Prompt PolyOrder, "Order of Fit ( 2 or greater )"
			Prompt ZeroYorN, "Do You Want to Force the Fit Through Zero Age at Zero Depth?", popup, "Yes;No"
			Prompt CursorYorN, "Do You Want to Fit only between Cursors A & B?", popup, "Yes;No"
			Prompt AgeOrDepth, "Do You Want Age or Depth as the X Axis?", popup, "Age;Depth"
			DoPrompt "SpectralAgeDepthFit", DepthWave, AgeWave, PolyOrder, ZeroYorN, CursorYorN, AgeOrDepth
			G_DepthWave = DepthWave; G_AgeWave = AgeWave; G_PolyOrder = PolyOrder
			G_ZeroYorN = ZeroYorN; G_CursorYorN = CursorYorN; G_AgeOrDepth = AgeOrDepth			
			AbortOnValue V_Flag, 0
			Print "SpectralAgeDepthFit_F( " + "\"" + Site + "\", \"" + Folder + "\", \"" + DepthWave + "\", \"" + AgeWave + "\", ", PolyOrder, ", \"" + ZeroYorN + "\", \"" + CursorYorN + "\", \"" + AgeOrDepth + "\" )"
			SpectralAgeDepthFit_F( Site, Folder, DepthWave, AgeWave, PolyOrder, ZeroYorN, CursorYorN, AgeOrDepth )
	End

----------------------------------------------------------------------

	Function SpectralAgeDepthFit_F( Site, Folder, DepthWave, AgeWave, PolyOrder, ZeroYorN, CursorYorN, AgeOrDepth )
			String Site, Folder, AgeWave, DepthWave, ZeroYorN, CursorYorN, AgeOrDepth
			Variable PolyOrder	
		SetDataFolder root:
			String/G G_Folder, G_AgeWave, G_DepthWave, G_ZeroYorN, G_Site, G_CursorYorN, G_AgeOrDepth
			Variable/G G_PolyOrder
			G_DepthWave = DepthWave; G_AgeWave = AgeWave; G_PolyOrder = PolyOrder
			G_ZeroYorN = ZeroYorN; G_CursorYorN = CursorYorN; G_AgeOrDepth = AgeOrDepth			
		SetDataFolder root:$Folder
			Wave DW = $DepthWave, AW = $AgeWave
			If( WaveType( DW, 1 ) == 2 || WaveType( AW, 1 ) == 2 ) // Check to see that the depth and age waves are numbers, not text
				SetDataFolder root:
				ABORT "Either " + AgeWave + " or " + DepthWave + " is a TEXT wave and can't be used for curve fitting.\r\rDelete the \"AgeDepthFit\" folder and run again."
			EndIf
			If( NumPnts( DW ) != NumPnts( AW ) ) // Check to see that depth and age waves are the same length
				SetDataFolder root:
				ABORT AgeWave + " does not have the same number of points as " + DepthWave
			EndIf
		NewDataFolder/O/S $"root:AgeDepthFit" // Shift to folder holding fit results
			Duplicate/O DW $Site + "_DepthWv" // Set up the depth, age, and weight waves for the fit
			Duplicate/O AW $Site + "_AgeWv"
			Duplicate/O AW $Site + "_Weight"
			WAVE DepthWv = $Site + "_DepthWv", AgeWv = $Site + "_AgeWv", Weight = $Site + "_Weight" // wave refs for above
			Weight = 1 // begin with all data weighted 1. weight of 0 excludes a point
			Make/T/O/N = ( NumPnts( DepthWv ) ) $Site + "_DataTypeWv" // Create the text wave to hold data types 
			WAVE/T DataTypeWv = $Site + "_DataTypeWv", MultiDataType = $Site + "_DataType_4_Fit" // wave ref for data type wave
//			If( WaveExists( MultiDataType ) ) // Multi data waves are those formed by AgeDataSelection() from > 1 data set
			If( WaveExists( MultiDataType ) && StringMatch( Folder, "AgeDepthFit" ) ) // Multi data waves are those formed by AgeDataSelection() from > 1 data set
				DataTypeWv = MultiDataType
			Else
				DataTypeWv = SecondPart( DepthWave )
			EndIf
   			If( StringMatch( ZeroYorN, "Yes" ) ) // Inserts 0 depth and 0 age point pair at top of data to be fit and forces fit through that point
				InsertPoints 0, 1, DepthWv, AgeWv, Weight, DataTypeWv
				DataTypeWv[0] = "Forced"
				Weight[ 0 ] = 10
				K0 = 0
			EndIf
			Sort DepthWv DepthWv, AgeWv, Weight, DataTypeWv
			If( StringMatch( CursorYorN, "No" ) )  // Don't use cursors
				If( StringMatch( AgeOrDepth, "Age" ))
					If( StrSearch( WinList( "*", ";", "Visible:1,WIN:1" ), "Poly_Fit_OnAge", 0 ) < 0 ) // Check to see if data are plotted already
						Display/N = Poly_Fit_OnAge /W = ( 431, 45, 952, 526 ) /B = AgeAxis /L = DepthAxis DepthWv vs AgeWv	as "Poly Fit On Age"	// Plot data	
					Else
						EXECUTE "DoWindow/F Poly_Fit_OnAge" // If data are already plotted, bring plot to front and remove fit and sed rate curves
					EndIf
				Else
					If( StrSearch( WinList( "*", ";", "Visible:1,WIN:1" ), "Poly_Fit_OnDepth", 0 ) < 0 ) // Check to see if data are plotted already
						Display/N = Poly_Fit_OnDepth /W = ( 431, 45, 952, 526 ) /B = AgeAxis /L = DepthAxis DepthWv vs AgeWv	as "Poly Fit On Depth"	// Plot data	
					Else
						EXECUTE "DoWindow/F Poly_Fit_OnDepth" // If data are already plotted, bring plot to front and remove fit and sed rate curves
					EndIf
				EndIf	
				If( StringMatch( ZeroYorN, "Yes" ) ) // No cursors and force fit through 0
					If( PolyOrder == 2 ) // Distinct command for linear fit
						CurveFit/Q/L=500/H="10" line, DepthWv /X=AgeWv /D /W = Weight
					Else // Command for all higher order fits
						CurveFit/Q/L=500/H="1000000000000" poly PolyOrder, DepthWv /X=AgeWv /D /W = Weight
					EndIf
				Else // No cursors and don't force fit through 0
					If( PolyOrder == 2 ) // Distinct command for linear fit
						CurveFit/Q/L=500 line, DepthWv /X=AgeWv /D /W = Weight		
					Else // Command for all higher order fits
						CurveFit/Q/L=500 poly PolyOrder, DepthWv /X=AgeWv /D /W = Weight
					EndIf
				EndIf
				WAVE W_coef			
			Else // Use Cursors
				If( StrSearch( WinList( "*", ";", "Visible:1,WIN:1" ), "Poly_Fit_OnAge", 0 ) < 0  && StrSearch( WinList( "*", ";", "Visible:1,WIN:1" ), "Poly_Fit_OnDepth", 0 ) < 0 )// If the data are not plotted, curors cannot be evaluated
					ABORT "In order to use the cursors the data must first be plotted.\r\rRun the function without cursors first, then add them and run again."
				EndIf		
				If( !StringMatch( CsrWave( A ), NameOfWave(DepthWv) ) ||  !StringMatch( CsrWave( B ), NameOfWave(DepthWv) ) ) // Check to see if cursors are on data points, not fit or sed rate curves
					ABORT "One or both Cursors are not on the data.\r\rPerhaps they are on the fit line instead?"
				EndIf
				If( StringMatch( ZeroYorN, "Yes" ) ) // Use cursors - Comments the same as above for no cursors
					If( PolyOrder == 2 )
						CurveFit/Q/L=500/H="10" line, DepthWv[ PCsr( A ), PCsr( B ) ] /X=AgeWv[ PCsr( A ), PCsr( B ) ] /D /W = Weight
					Else
						CurveFit/Q/L=500/H="1000000000000" poly PolyOrder, DepthWv[ PCsr( A ), PCsr( B ) ] /X=AgeWv[ PCsr( A ), PCsr( B ) ] /D /W = Weight
					Endif
				Else
					If( PolyOrder == 2 )
						CurveFit/Q/L=500 line, DepthWv[ PCsr( A ), PCsr( B ) ] /X=AgeWv[ PCsr( A ), PCsr( B ) ] /D /W = Weight		
					Else
						CurveFit/Q/L=500 poly PolyOrder, DepthWv[ PCsr( A ), PCsr( B ) ] /X=AgeWv[ PCsr( A ), PCsr( B ) ] /D /W = Weight
					EndIf
				EndIf
				WAVE W_coef		
			EndIf
			Duplicate/O W_coef $"fit_" + Site + "_Coefs" // Save the coefficient wave with unique name
			Duplicate/O $"fit_" + Site + "_DepthWv" $"fit_" + Site + "_AgeWv", $Site + "_SedRateDepth" // Make an age wave for the fit and a depth wave for the sed rate
			WAVE AgeFit = $"fit_" + Site + "_AgeWv", DepthFit = $"fit_" + Site + "_DepthWv" // wave refs for above
			WAVE SedRateDepth = $Site + "_SedRateDepth"
			If( CheckAgeModel( AgeFit, DepthFit ) == 0 ) // Warn user if the fit curve does not increase monotonically
				DoAlert/T="Age Model Warning", 0, "The age/depth fit is recurved ( negative sed rates ).\r\rThis may cause a problem when converting depth to age.\r\r Either rerun with a different Order of fit or hand edit the fit curves."
			EndIf
			Differentiate DepthFit /D = $Site + "_SedRate" // The differential of the age/depth fit is the sed rate
			WAVE SedRate = $Site + "_SedRate"
			If( PolyOrder == 2 )
				SedRate = Round( SedRate ) // For linear fits the constant sedimentation rate is rounded off
			EndIf
			If( StrSearch( TraceNameList( "", ";", 1 ), "SedRateDepth", 0 ) < 0 ) // Check to see if the sed rate is already plotted on the graph
				If(1)
						AppendToGraph/T = SedRateAxis/L = DepthAxis SedRateDepth vs SedRate // add sed rate on its own top axis
				Else
				
				EndIf
				Label SedRateAxis "SedRate m/Myr";DelayUpdate
				ModifyGraph axisEnab(SedRateAxis)={0.5,1}
				ModifyGraph rgb($NameOfWave(SedRateDepth))=(65535,43690,0)
			EndIf 
			AgeFit = x
			SetAxis AgeAxis 0,*
			SetAxis/R DepthAxis *,0
			ModifyGraph mode($Site + "_DepthWv")=3,marker($Site + "_DepthWv")=19,msize($Site + "_DepthWv")=7;DelayUpdate
			ModifyGraph mrkThick($Site + "_DepthWv")=3, lsize = 2
			ModifyGraph freePos = 0
			ModifyGraph zColor( $Site + "_DepthWv" )={$Site + "_Weight",0,10,Rainbow16,1}
			Label DepthAxis "Depth";DelayUpdate
			Label AgeAxis "Age"
			ModifyGraph lblPos = 50
			ModifyGraph lblPosMode = 4
			ShowInfo
			Set_Cursors_Red()
			KillWaves/Z W_ParamConfidenceInterval, W_sigma, W_coef
			If( StringMatch( "Forced", DataTypeWv[ 0 ] ) )
				Weight[ 0 ] = 10
			EndIf
			Duplicate/O/T DataTypeWv Temp
			Sort Temp Temp
			String DataTypeList = ( Temp[ 0 ] + ";" ) // start the data type list with the first data type
			Variable i, j, NumberOfTypes = ItemsInList( DataTypeList )
			For( i = 0; i < NumPnts( Temp ); i += 1 )
				NumberOfTypes = ItemsInList( DataTypeList )				
				If( !StringMatch( Temp[ i ], StringFromList( NumberOfTypes - 1, DataTypeList ) ) )
					DataTypeList = DataTypeList + Temp[ i ] + ";"					
				EndIf				
			EndFor
			Killwaves/Z Temp
			String TypeString4Legend = ""		
			Duplicate/O DepthWv $Site + "_Markers"
			WAVE MarkerWv  = $Site + "_Markers"
			MarkerWv = 0
			For( i = 0; i < ItemsInList( DataTypeList ); i += 1 )
				MarkerWv = StringMatch( StringFromList( i, DataTypeList ), DataTypeWv ) ? i : MarkerWv
				TypeString4Legend = TypeString4Legend + "\Z14\\W" + Num2Str( 8000 + i ) + " " + StringFromList( i, DataTypeList ) + "\r"
			EndFor
				// Add a color bar to the plot using the Rainbow16 color scale
			ColorScale/C/N=ColorText/F=0/B=1/A=LB vert=0,widthPct=40,colorBoxesFrame=1,trace=$NameOfWave( DepthWv )
			ColorScale/C/N=ColorText fsize=12,fstyle=1,nticks=10;DelayUpdate
			ColorScale/C/N=ColorText "Weight"		
			ModifyGraph zmrkNum( $NameOfWave( DepthWv ) ) = { $Site + "_Markers" } // Set different data types tom different symbols			
		SetDataFolder root:
			If( StrSearch( WinList( "*", ";", "Visible:1" ), "FittingData", 0 ) < 0 ) // Check to see if the data table is already open
				Edit/N = FittingData AgeWv, DepthWv, DataTypeWv, Weight, MarkerWv
				ModifyTable alignment=1
			EndIf			
			EXECUTE "DoWindow/F FittingData"	// Move the data table to the front	
			Legend/C/N=LegendText/J/B=1/F=0/A=MT 	TypeString4Legend
			CalculatePolyPredictions_F( Site )
			If( StringMatch( AgeOrDepth, "Depth" ) )
				DoWindow/Z/F Poly_Fit_OnDepth				
				ModifyGraph swapXY = 1
				ModifyGraph axisEnab(SedRateAxis)={0.25,0.75}
			EndIf		
			If( StrSearch( WinList( "*", ";", "Visible:1,WIN:1" ), "Poly_Fit_OnDepth", 0 ) >= 0 ) // Check to see if data are plotted already
				DoWindow/Z/F Poly_Fit_OnDepth
				SetAxis DepthAxis *,*
				SetAxis/R AgeAxis *,*
			EndIf
			If( StrSearch( WinList( "*", ";", "Visible:1,WIN:1" ), "Poly_Fit_OnAge", 0 ) >= 0 ) // Check to see if data are plotted already
				DoWindow/Z/F Poly_Fit_OnAge	
				SetAxis/R DepthAxis *,*
				SetAxis AgeAxis *,*
			EndIf
			DoWindow/H
	End

	---------------------------------------------------------------

//	Function SpectralSetWeights()
		SetDataFolder root:AgeDepthFit
			String/G G_Site
			String Site = G_Site, DataTypeList
			Prompt Site, "Site"
			DoPrompt "SetWeights", Site;
			G_Site = Site
			AbortOnValue V_Flag, 0			
			Wave/T DataTypeWv = $Site + "_DataTypeWv" // ref for data type wave
			Wave WeightWave = $Site + "_Weight" // ref for data weighting wave
			Duplicate/O/T DataTypeWv Temp
			Sort Temp Temp
			DataTypeList = ( Temp[ 0 ] + ";" ) // start the data type list with the first data type
			Variable i, j, NumberOfTypes = ItemsInList( DataTypeList )
			For( i = 0; i < NumPnts( Temp ); i += 1 )
				NumberOfTypes = ItemsInList( DataTypeList )				
				If( !StringMatch( Temp[ i ], StringFromList( NumberOfTypes - 1, DataTypeList ) ) )
					DataTypeList = DataTypeList + Temp[ i ] + ";"					
				EndIf				
			EndFor
			String/G G_DataType
			String DataType = G_DataType
			Variable/G G_WeightValue
			Variable WeightValue = G_WeightValue
			Prompt DataType, "Select the data type for weight change", popup, DataTypeList
			Prompt WeightValue, "Enter the new weight"
			DoPrompt "SetWeights", DataType, WeightValue
			G_DataType = DataType, G_WeightValue = WeightValue
			AbortOnValue V_Flag, 0
			WeightWave = StringMatch( DataType, DataTypeWv ) ? WeightValue : WeightWave
			KillWaves/Z Temp
		SetDataFolder root:
	End


	---------------------------------------------------------------

//	Function SpectralSetWeights_F()
		SetDataFolder root:AgeDepthFit
			String/G G_Site
			String Site = G_Site, DataTypeList
			Prompt Site, "Site"
			DoPrompt "SetWeights", Site;
			G_Site = Site
			AbortOnValue V_Flag, 0			
			Wave/T DataTypeWv = $Site + "_DataTypeWv" // ref for data type wave
			Wave WeightWave = $Site + "_Weight" // ref for data weighting wave
			Duplicate/O/T DataTypeWv Temp
			Sort Temp Temp
			DataTypeList = ( Temp[ 0 ] + ";" ) // start the data type list with the first data type
			Variable i, j, NumberOfTypes = ItemsInList( DataTypeList )
			For( i = 0; i < NumPnts( Temp ); i += 1 )
				NumberOfTypes = ItemsInList( DataTypeList )				
				If( !StringMatch( Temp[ i ], StringFromList( NumberOfTypes - 1, DataTypeList ) ) )
					DataTypeList = DataTypeList + Temp[ i ] + ";"					
				EndIf				
			EndFor
			String/G G_DataType
			String DataType = G_DataType
			Variable/G G_WeightValue
			Variable WeightValue = G_WeightValue
			Prompt DataType, "Select the data type for weight change", popup, DataTypeList
			Prompt WeightValue, "Enter the new weight"
			DoPrompt "SetWeights", DataType, WeightValue
			G_DataType = DataType, G_WeightValue = WeightValue
			AbortOnValue V_Flag, 0
			WeightWave = StringMatch( DataType, DataTypeWv ) ? WeightValue : WeightWave
			KillWaves/Z Temp
		SetDataFolder root:
	End

-------------------------------------------------------

//	Function Weight_0()
		SetDataFolder root:AgeDepthFit
			Wave WeightWave = $FirstPart( CsrWave( A ) ) + "_Weight"
			WeightWave[ PCsr( A ) ] = 0
		SetDataFolder root:
	End	
	
-------------------------------------------------------

